//===============================================================================
// Copyright 2014 Alachisoft.  All rights reserved.
//===============================================================================

using System;
using System.Collections;
using System.Collections.Generic;

using Alachisoft.NCache.Caching.AutoExpiration;
using Alachisoft.NCache.Caching.EvictionPolicies;
using Alachisoft.NCache.Runtime.DatasourceProviders;
using Alachisoft.NCache.Runtime.Caching;

using Directory.Business;
using Directory.Utility;

namespace Directory.Providers
{
	/// <summary>
	/// Contains methods used to read an object by its key from the master data source. 
	/// 
	/// For more information on how to setup databases like Oracle and IBM DB2 for 
	/// item invalidation notifications, please refer to NCache help collection.
	/// 
	/// </summary>
	public class OleDbReadThruProvider : Alachisoft.NCache.Runtime.DatasourceProviders.IReadThruProvider
	{
		private OleDbDatasource oledbDatasource;

		/// <summary>
		/// Responsible for loading the object from the external data source. 
		/// Key is passed as parameter.
		/// <param name="key">item identifier; probably a primary key</param>
		/// <param name="exh">Current expiration hint; you can modify the value and attach a new hint</param>
		/// <param name="evh">Current eviction hint; you can modify the value and attach a new hint</param>
		/// <returns></returns>
		public void LoadFromSource(string key, out ProviderCacheItem cacheItem)
		{
			cacheItem = new ProviderCacheItem(oledbDatasource.LoadCustomer(key));
		}

		/// <summary>
		/// Perform tasks like allocating resources or acquiring connections
		/// </summary>
		/// <param name="parameters">Startup paramters defined in the configuration</param>
        /// <param name="cacheId">Define for which cache provider is configured</param>
		public void Init(IDictionary parameters, string cacheId)
		{
			object connString = parameters["connstring"];
			oledbDatasource = new OleDbDatasource();
			oledbDatasource.Connect( connString == null ? "" : connString.ToString() );
		}

		/// <summary>
		///  Perform tasks associated with freeing, releasing, or resetting resources.
		/// </summary>
		public void Dispose()
		{
			oledbDatasource.DisConnect();
		}

        #region IReadThruProvider Members

        public Dictionary<string, ProviderCacheItem> LoadFromSource(string[] keys)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
}
