//===============================================================================
// Copyright 2014 Alachisoft.  All rights reserved.
//===============================================================================

using System;
using System.Collections;

using Alachisoft.NCache.Caching.AutoExpiration;
using Alachisoft.NCache.Caching.EvictionPolicies;
using Alachisoft.NCache.Runtime.DatasourceProviders;

using Directory.Business;
using Directory.Utility;

namespace Directory.Providers
{
	/// <summary>
	/// Contains methods used to save/update an object to the master data source. 
	/// 
	/// For more information on how to setup databases like Oracle and IBM DB2 for 
	/// item invalidation notifications, please refer to NCache help collection.
	/// 
	/// </summary>
	public class OleDbWriteThruProvider : Alachisoft.NCache.Runtime.DatasourceProviders.IWriteThruProvider
	{
		/// <summary>
		/// Object used to communicate with the Datasource.
		/// </summary>
		OleDbDatasource oledbDatasource;
		
		/// <summary>
		/// Perform tasks like allocating resources or acquiring connections
		/// </summary>
		/// <param name="parameters">Startup paramters defined in the configuration</param>
        /// <param name="cacheId">Define for which cache provider is configured</param>
		public void Init(IDictionary parameters, string cacheId)
		{
			object connString = parameters["connstring"];
			oledbDatasource = new OleDbDatasource();
			oledbDatasource.Connect( connString == null ? "" : connString.ToString() );
		}

		/// <summary>
		/// Perform tasks associated with freeing, releasing, or resetting resources.
		/// </summary>
		public void Dispose()
		{
			oledbDatasource.DisConnect();
		}

        #region IWriteThruProvider Members

        public OperationResult WriteToDataSource(WriteOperation operation)
        {
            bool result = false;
            OperationResult operationResult = new OperationResult(operation, OperationResult.Status.Failure);
            Customer value = (Customer)operation.ProviderCacheItem.Value;
            if (value.GetType().Equals(typeof(Customer)))
            {
                result = oledbDatasource.SaveCustomer((Customer)value);
            }
            if (result) operationResult.DSOperationStatus = OperationResult.Status.Success;
            return operationResult;
        }

        public OperationResult[] WriteToDataSource(WriteOperation[] operation)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
}
