//===============================================================================
// Copyright 2014 Alachisoft.  All rights reserved.
//===============================================================================

using System;
using System.IO;
using System.Data;
using Microsoft.Win32;
using System.Windows.Forms;

using Directory.Business;

namespace Directory.Utility
{
	/// <summary>
	/// Class that helps read and write customer information to an xml
	/// datasource.
	/// </summary>
	internal class XmlDataSource
	{
		private static DataSet _ds = null;
        private static string DsFilePath = @"Samples\clr40\Directory\Data\northwind.db.xml";

		static XmlDataSource()
		{
			try
			{
				RegistryKey key = Registry.LocalMachine.OpenSubKey(@"Software\Alachisoft\NCache");
				DsFilePath = Path.Combine(key.GetValue("InstallDir").ToString(),DsFilePath);
			}
			catch(Exception e)
			{
				System.Diagnostics.Trace.WriteLine(e);
			}
		}

		/// <summary>
		/// Establish connection with the datasource.
		/// </summary>
		/// <param name="connString"></param>
		static public void Connect()
		{
			if(_ds != null)
				return;
			_ds = new DataSet("Northwind");
			if(!File.Exists(DsFilePath))
			{
				DataTable dt = new DataTable("Customer");

				dt.Columns.Add("CustomerID", typeof(string));
				dt.Columns.Add("ContactName", typeof(string));
				dt.Columns.Add("CompanyName", typeof(string));
				dt.Columns.Add("Address", typeof(string));
				dt.Columns.Add("City", typeof(string));
				dt.Columns.Add("Country", typeof(string));
				dt.Columns.Add("Postal_Code", typeof(string));
				dt.Columns.Add("Phone", typeof(string));
				dt.Columns.Add("Fax", typeof(string));
				dt.PrimaryKey =  new DataColumn[] { dt.Columns[0] };
			
				_ds.Tables.Add(dt);
				_ds.WriteXml(DsFilePath, XmlWriteMode.WriteSchema);
			}
			else
			{
				_ds.ReadXml(DsFilePath, XmlReadMode.Auto);
			}
		}

		/// <summary>
		/// Releases the connection.
		/// </summary>
		static public void DisConnect()
		{
			_ds = null;
		}

		/// <summary>
		/// Loads a <see cref="Customer"/> object from the datasource.
		/// </summary>
		/// <param name="key"></param>
		/// <returns></returns>
		static public object LoadCustomer(Object key)
		{			
			DataRow dr = _ds.Tables["Customer"].Rows.Find(key);
			if(dr!=null)
			{
				Customer objCustomer = new Customer();

				objCustomer.CustomerID	= dr["CustomerID"].ToString();
				objCustomer.ContactName = dr["ContactName"].ToString();
				objCustomer.CompanyName = dr["CompanyName"].ToString();
				objCustomer.Address		= dr["Address"].ToString();
				objCustomer.City		= dr["City"].ToString();
				objCustomer.Country		= dr["Country"].ToString();
				objCustomer.PostalCode	= dr["Postal_Code"].ToString();
				objCustomer.Phone		= dr["Phone"].ToString();
				objCustomer.Fax			= dr["Fax"].ToString();
				
				return objCustomer;
			}
			return null;
		}

		/// <summary>
		/// Save <see cref="Customer"/> information to datasource.
		/// </summary>
		/// <param name="val"></param>
		/// <returns></returns>
		static public bool SaveCustomer(Customer val)
		{
			try
			{
				DataRow dr = _ds.Tables["Customer"].Rows.Find(val.CustomerID);

				if(dr == null)
				{
					dr = _ds.Tables["Customer"].NewRow();
				}
				
				dr["CustomerID"]	= val.CustomerID;
				dr["ContactName"]	= val.ContactName;
				dr["CompanyName"]	= val.CompanyName;
				dr["Address"]		= val.Address;
				dr["City"]			= val.City;
				dr["Country"]		= val.Country;
				dr["Postal_Code"]	= val.PostalCode;
				dr["Phone"]			= val.Phone;
				dr["Fax"]			= val.Fax;

				if(dr.RowState == DataRowState.Detached)
					_ds.Tables["Customer"].Rows.Add(dr);
				else
					dr.AcceptChanges();

				_ds.WriteXml(DsFilePath, XmlWriteMode.WriteSchema);
			}
			catch(Exception e)
			{
				System.Diagnostics.Trace.WriteLine(e);
                return false;
			}
			return true;
		}
	}
}
